#include <algorithm>

#include "rgb2hsl.hpp"

static inline void rgb_to_hsl(unsigned red, unsigned green, unsigned blue,
             double & h, double & s, double & l) {
    double r = red/255.0;
    double g = green/255.0;
    double b = blue/255.0;
    double max = std::max(r,std::max(g,b));
    double min = std::min(r,std::min(g,b));
    double delta = max - min;
    double gamma = max + min;
    h = 0.0, s = 0.0, l = gamma / 2.0;
    if (delta > 0.0) {
        s = l > 0.5 ? delta / (2.0 - gamma) : delta / gamma;
        if (max == r && max != g) h = (g - b) / delta + (g < b ? 6.0 : 0.0);
        if (max == g && max != b) h = (b - r) / delta + 2.0;
        if (max == b && max != r) h = (r - g) / delta + 4.0;
        h /= 6.0;
    }
}

void rgb2hsl(unsigned rgb[3], double hsl[3])
{
    unsigned r,g,b;
    r = rgb[0];
    g = rgb[1];
    b = rgb[2];
    double h,s,l;
    rgb_to_hsl(r,g,b,h,s,l);
    hsl[0] = h;
    hsl[1] = s;
    hsl[2] = l;
    return;
}

